=== modified file 'lib/lp/app/browser/tests/test_vocabulary.py'
--- lib/lp/app/browser/tests/test_vocabulary.py	2011-09-20 20:33:47 +0000
+++ lib/lp/app/browser/tests/test_vocabulary.py	2011-09-26 19:11:51 +0000
@@ -224,6 +224,18 @@
             distroarchseries=archseries)
         self.assertEqual("fnord", self.getPickerEntry(dsp).description)
 
+    def test_dsp_provides_alt_title_link(self):
+        distro = self.factory.makeDistribution(name='fnord')
+        series = self.factory.makeDistroSeries(
+            name='pting', distribution=distro)
+        self.factory.makeSourcePackage(
+            sourcepackagename='snarf', distroseries=series, publish=True)
+        dsp = distro.getSourcePackage('snarf')
+        self.assertEqual(
+            'http://launchpad.dev/fnord/+source/snarf',
+            self.getPickerEntry(dsp).alt_title_link)
+
+
 class TestProductPickerEntrySourceAdapter(TestCaseWithFactory):
 
     layer = DatabaseFunctionalLayer
@@ -275,6 +287,12 @@
         self.assertEqual(
             expected_details, entry.details[0])
 
+    def test_product_provides_alt_title_link(self):
+        product = self.factory.makeProduct(name='fnord')
+        self.assertEqual(
+            'http://launchpad.dev/fnord',
+            self.getPickerEntry(product).alt_title_link)
+
 
 class TestProjectGroupPickerEntrySourceAdapter(TestCaseWithFactory):
 
@@ -328,6 +346,13 @@
         self.assertEqual(
             expected_details, entry.details[0])
 
+    def test_projectgroup_provides_alt_title_link(self):
+        projectgroup = self.factory.makeProject(name='fnord')
+        self.assertEqual(
+            'http://launchpad.dev/fnord',
+            self.getPickerEntry(projectgroup).alt_title_link)
+
+
 class TestDistributionPickerEntrySourceAdapter(TestCaseWithFactory):
 
     layer = DatabaseFunctionalLayer
@@ -372,7 +397,7 @@
     def test_distribution_truncates_summary(self):
         summary = ("This is a deliberately, overly long summary. It goes on"
                    "and on and on so as to break things up a good bit.")
-        distribution= self.factory.makeDistribution(summary=summary)
+        distribution = self.factory.makeDistribution(summary=summary)
         index = summary.rfind(' ', 0, 45)
         expected_summary = summary[:index + 1]
         expected_details = summary[index:]
@@ -382,6 +407,13 @@
         self.assertEqual(
             expected_details, entry.details[0])
 
+    def test_distribution_provides_alt_title_link(self):
+        distribution = self.factory.makeDistribution(name='fnord')
+        self.assertEqual(
+            'http://launchpad.dev/fnord',
+            self.getPickerEntry(distribution).alt_title_link)
+
+
 class TestPersonVocabulary:
     implements(IHugeVocabulary)
     test_persons = []

=== modified file 'lib/lp/app/browser/vocabulary.py'
--- lib/lp/app/browser/vocabulary.py	2011-09-20 20:34:30 +0000
+++ lib/lp/app/browser/vocabulary.py	2011-09-26 19:45:51 +0000
@@ -260,7 +260,7 @@
                 picker_entry.details = []
                 summary = picker_entry.description
                 if len(summary) > 45:
-                    index =  summary.rfind(' ', 0, 45)
+                    index = summary.rfind(' ', 0, 45)
                     first_line = summary[0:index + 1]
                     second_line = summary[index:]
                 else:
@@ -268,15 +268,17 @@
                     second_line = ''
 
                 if len(second_line) > 90:
-                    index =  second_line.rfind(' ', 0, 90)
-                    second_line = second_line[0:index+1] 
+                    index = second_line.rfind(' ', 0, 90)
+                    second_line = second_line[0:index + 1]
                 picker_entry.description = first_line
                 picker_entry.details.append(second_line)
                 picker_entry.alt_title = target.name
+                picker_entry.alt_title_link = canonical_url(
+                    target, rootsite='mainsite')
                 picker_entry.target_type = self.target_type
                 maintainer = self.getMaintainer(target)
                 if maintainer is not None:
-                    picker_entry.details.append( 
+                    picker_entry.details.append(
                         'Maintainer: %s' % self.getMaintainer(target))
         return entries
 

